#! /bin/sh

# 
# Copyright (c) 2014 Benedikt Stockebrand
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# 
# 


# Configurables
IMAGEDIR=/vol/InstallImages

# This should also work out using plain /usr/bin/dialog on FBSD
DIALOG="`which dialog || which whiptail`"

# Simplify the next operations
cd "$IMAGEDIR"

# If $image.restrict returns false, don't offer this image.
for i in `ls *.img *.img.gz *.img.bz2 *.img.xz *.img.lzo 2>/dev/null`
do
    if [ -f $i ]
    then
        if ! [ -f $i.restrict ] || sh $i.restrict
        then
            imageargs="$imageargs $i off"
        fi
    fi
done

# Figure out which image to install
image="`$DIALOG 2>/dev/stdout >/dev/tty \
                 --noitem --menu 'Choose image:' 20 70 12 \
                 '***   Reboot   ***' on \
                 '*** Erase disk ***' off \
                 $imageargs`"

# If we want to reboot, we don't need to choose a disk
# and if we cancel, we just restart.
case "$image" in
  "***   Reboot   ***")  reboot; sleep 10000 ;;
  '')                    exec $0 ;;
esac

# Find all disks, turn them into a selection for whiptail/dialog
dsts="`ls /dev/hd[a-z] /dev/sd[a-z] /dev/xvd[a-z] 2>/dev/null`"
dsts="`echo $dsts | tr ' ' '\n' | sed '1s/$/ on/;2,$s/$/ off/'`"

# Choose the disk
disk="`$DIALOG 2>/dev/stdout >/dev/tty \
                 --noitem --menu \"Install $image on:\" 20 70 12 \
                 $dsts`"

# Check if the operation was cancelled
[ "$disk" != "" ] || exec $0

# Figure out the extract command according to the image file's format
case "$image" in
  *Erase\ disk*)  cmd="dd if=/dev/zero of=$disk bs=1024k" ;;
  *.img)          cmd="dd if=$image of=$disk bs=1024k" ;;
  *.img.gz)       cmd="gzip -d <$image | dd of=$disk bs=1024k" ;;
  *.img.bz2)      cmd="bzip2 -d <$image | dd of=$disk bs=1024k" ;;
  *.img.xz)       cmd="xz -d <$image | dd of=$disk bs=1024k" ;;
  *.img.lzo)      cmd="lzop -d <$image | dd of=$disk bs=1024k" ;;
esac

# Execute command and announce success/failure.
#
# Is there a reasonable way to make the erase operation claim success
# if the entire disk was wiped?
echo "Executing '$cmd'"
echo
if   sh -c "$cmd"
then echo "Operation completed."
else echo "Operation failed (that's ok for erasing a disk)."
fi
echo Please press the Anykey.
read junk
